using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Threading;
using iNTERFACEWARE.Chameleon;
using PrefixTest;

/// <summary>
/// This application is a simple example of using the Chameleon generated objects.  It
/// is intended to be a helpful starting point in understanding how to use these objects
/// but must be fleshed out in the following areas to provide a full solution for an
/// application HL7 interface:
/// 1) You will need to work out how your application will handling logging of messages
///    and data, in particular how bad transactions should be logged for effective troubleshooting.
/// 2) The code will need to be modified to insert the parsed data into a database whether
///    by using explicit database calls or going through some type of middle tier layer that
///    your application may support.
/// 3) Some customers choose to deploy HL7 interfaces as Windows Services.  This can be done
///    easily with Chameleon.  One thing to keep in mind though is that the networking components
///    supplied with Chameleon require a Windows 'message loop'.  Often Windows Services do
///    not have a message loop - but one can be easily set up by invoking RunMessageLoop on
///    the networking component supplied with Chameleon.  See the manual for running Chameleon
///    within services.
/// 4) Definitely take the time to read the manual and go through the tutorials after looking
///    at this example before attempting to deploy a production solution.  We do provide custom
///    on-site training and technical implementation services if you require them.
///
/// If you are looking for a much more complete 'out of the box' solution for your HL7 needs it would be
/// a good idea to evaluate iNTERFACEWARE Iguana.  Iguana is powered by Chameleon but
/// provides a complete run-time web-based infrastructure for your HL7 interfaces.
/// It provides advanced logging and monitoring capability which makes it a very attractive option 
/// for many customers.
/// </summary>
public class ServerForm : System.Windows.Forms.Form, Log
{
	private const ushort DEFAULT_PORT = 5123;
  	private const int MAX_BUFFER = 30000;
	private string MESSAGE_DEFINITION_FILE = "test.vmd";
	private string AUTOACK_DEFINITION_FILE = "autoack.vmd";
	private string AUTONACK_DEFINITION_FILE = "autonack.vmd";
	
	private iNTERFACEWARE.Chameleon.DynamicEngine ackEngine;
	private ServerEngine engine;
	private iNTERFACEWARE.Chameleon.DynamicEngine nackEngine;
	private iNTERFACEWARE.Chameleon.LlpServer llpServer;
	/// <summary>
	/// Required designer variable.
	/// </summary> 
	private System.ComponentModel.Container components = null;
	private System.Windows.Forms.Panel _middlePanel;
	private System.Windows.Forms.Label _portLabel;
	private System.Windows.Forms.TextBox _portTextBox;
	private System.Windows.Forms.Button _startStopListenBtn;
	private System.Windows.Forms.TextBox _logTextArea;
	private System.Windows.Forms.Panel _topPanel;
	private System.Windows.Forms.Label _configurationLabel;
	private System.Windows.Forms.ComboBox _configurationCmbBox;
	private System.Windows.Forms.Panel _bottomPanel;
	
	static public void Main(String[] args)
	{
		ServerForm FormMain = new ServerForm();
		FormMain.ShowDialog();
	}

	public ServerForm()	
	{	  
		// Required for Windows Form Designer support		 
		InitializeComponent();
	}

	private void ServerForm_Load(object sender, System.EventArgs e)
	{	
		// Initialize engine
		try
		{
			engine = new ServerEngine();
			engine.AddLog(this);

            // In order to make the engine useable we must load the vmd file.  This file
            // contains the information about the structure of the messages, tables, and data
            // mappings that were defined in the Chameleon GUI.  ServerEngine() is a generated 
            // Engine which has an interface based on the messages and tables defined in the vmd file.
            // 
            // Also keep in mind that this example is using the static stubcode 
            // interface of Chameleon. For advanced applications it is possible to 
            // use a dynamic interface to populate messages.  See the manual for 
            // more details.
			engine.Load(MESSAGE_DEFINITION_FILE);
			Println("Loaded " + MESSAGE_DEFINITION_FILE);
			
			// We also initialize the engine used for auto-acknowledgement/negative acknowledgement.
			// We use 2 more engines to generate HL7 acknowledgement (ACK) and negative 
			// acknowlegement (NACK) messages. Here the Engines we use are DynamicEngine 
			// objects. 
			ackEngine = new iNTERFACEWARE.Chameleon.DynamicEngine();
			ackEngine.Load(AUTOACK_DEFINITION_FILE);
			Println("Loaded " + AUTOACK_DEFINITION_FILE);
			
			nackEngine = new iNTERFACEWARE.Chameleon.DynamicEngine();
			nackEngine.Load(AUTONACK_DEFINITION_FILE );
			Println("Loaded "+ AUTONACK_DEFINITION_FILE + "\r\n"); 
		}
		catch (ChameleonException ex)
		{
			Println(engine.MakeMessagePrintable(ex.ToString() + " while trying to initialize engine"));
		}		
		// This example includes a combo box that allows the user to switch between configurations.
		// For more information regarding multiple configurations, please see the Chameleon manual.
		try
		{
			for (int ConfigurationIndex = 0; 
				ConfigurationIndex <= engine.CountOfConfig() - 1; 
				++ConfigurationIndex)
			{
				_configurationCmbBox.Items.Add(engine.ConfigName(ConfigurationIndex));
			}		
			_configurationCmbBox.SelectedIndex=0;
		}
		catch (ChameleonException ex)
		{
			Println(engine.MakeMessagePrintable(ex.ToString() + " while trying to build configuration list"));
		}
	}

	private void ConfigurationSelectedIndexChanged(object sender, System.EventArgs e)
	{
		// Change the current engine configuration to the selected configuration in the list
		engine.CurrentConfigIndex = _configurationCmbBox.SelectedIndex;
		Println("Configuration set to " + engine.ConfigName(engine.CurrentConfigIndex) + "\r\n");
	}

	// This method stops the server on the current port and restarts it on the port 
	// specified in the textbox.  It is just part of mechanics of this example.
	private void ChangeServerState()
	{
		ushort port;
		try 
		{
			port = System.Convert.ToUInt16(_portTextBox.Text);
		}
		catch
		{		
			Println(engine.MakeMessagePrintable("Invalid port number entered - Default port " + DEFAULT_PORT + " used"));
			port = DEFAULT_PORT;
			_portTextBox.Text = port.ToString();
		}
		  
		if (llpServer.IsListening)
		{
			Println("Stop listening on current port\r\n" );
			// This method stops the listener from listening on the current port
			llpServer.StopListen();	
			llpServer.DisconnectAllClients();
			// This method starts the listener to listen on 'Port'
			this._startStopListenBtn.Text = "&Start Listening";
		}
		else
		{
			llpServer.StartListen(port);
			Println("Listening on port " + port + "\r\n");		
			this._startStopListenBtn.Text = "&Stop Listening";
		}
	}
		
	private void StartStopListenBtnClicked(object sender, System.EventArgs e)	
	{		
      try
      {
         ChangeServerState();
      }
      catch(ChameleonException ex)
      {
         Println(llpServer.MakeMessagePrintable(ex.Message,"\r\n",-1 ) );
      }
      catch
      {
         Println("Unhandled exception while connecting to port " + _portTextBox.Text );
      }
	}
	
	/// <summary>
	/// Clean up any resources being used.
	/// </summary>
	protected override void Dispose(bool disposing)
	{
		if(disposing)
		{
			if(components != null)
			{
				components.Dispose();
			}
		}
		base.Dispose(disposing);
	}

	// This code is only for demonstration purposes.  Memo controls can only take a certain amount
	// of data - 32k at the time this example was written.  As a result we check the length of
	// the memo control and truncate it if it reaches this limit.

	// It should be strongly emphasized that this solution is only for demonstration purposes.
	// A real HL7 interface should have some type of system of logging into log files.  Iguana for
	// instance has a very nice logging system that allows administrators to drill into and browse
	// processed messages and errors.  How you implement this logging is up to your requirements
   	// for run time management of your HL7 interface.

	public void Println(string line)
	{
      if(_logTextArea.Text.Length > MAX_BUFFER )
      {
         _logTextArea.Text = _logTextArea.Text.Substring(_logTextArea.Text.Length/2,_logTextArea.Text.Length/2);
      }
      _logTextArea.AppendText(line + "\r\n");
   }

#region Windows Form Designer generated code
	/// <summary>
	/// Required method for Designer support - do not modify
	/// the contents of this method with the code editor.
	/// </summary>
	private void InitializeComponent()
	{
      this._bottomPanel = new System.Windows.Forms.Panel();
      this._portLabel = new System.Windows.Forms.Label();
      this._portTextBox = new System.Windows.Forms.TextBox();
      this._startStopListenBtn = new System.Windows.Forms.Button();
      this._middlePanel = new System.Windows.Forms.Panel();
      this._logTextArea = new System.Windows.Forms.TextBox();
      this._topPanel = new System.Windows.Forms.Panel();
      this._configurationLabel = new System.Windows.Forms.Label();
      this._configurationCmbBox = new System.Windows.Forms.ComboBox();
      this.llpServer = new iNTERFACEWARE.Chameleon.LlpServer();
      this._bottomPanel.SuspendLayout();
      this._middlePanel.SuspendLayout();
      this._topPanel.SuspendLayout();
      this.SuspendLayout();
      // 
      // _bottomPanel
      // 
      this._bottomPanel.Controls.AddRange(new System.Windows.Forms.Control[] {
                                                                                this._portLabel,
                                                                                this._portTextBox,
                                                                                this._startStopListenBtn});
      this._bottomPanel.Dock = System.Windows.Forms.DockStyle.Bottom;
      this._bottomPanel.Location = new System.Drawing.Point(0, 313);
      this._bottomPanel.Name = "_bottomPanel";
      this._bottomPanel.Size = new System.Drawing.Size(542, 56);
      this._bottomPanel.TabIndex = 2;
      // 
      // _portLabel
      // 
      this._portLabel.Location = new System.Drawing.Point(8, 16);
      this._portLabel.Name = "_portLabel";
      this._portLabel.Size = new System.Drawing.Size(40, 24);
      this._portLabel.TabIndex = 0;
      this._portLabel.Text = "Port:";
      this._portLabel.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
      // 
      // _portTextBox
      // 
      this._portTextBox.Location = new System.Drawing.Point(56, 16);
      this._portTextBox.MaxLength = 5;
      this._portTextBox.Name = "_portTextBox";
      this._portTextBox.Size = new System.Drawing.Size(88, 20);
      this._portTextBox.TabIndex = 1;
      this._portTextBox.Text = "5123";
      // 
      // _startStopListenBtn
      // 
      this._startStopListenBtn.Location = new System.Drawing.Point(200, 16);
      this._startStopListenBtn.Name = "_startStopListenBtn";
      this._startStopListenBtn.Size = new System.Drawing.Size(128, 24);
      this._startStopListenBtn.TabIndex = 2;
      this._startStopListenBtn.Text = "&Start Listening";
      this._startStopListenBtn.Click += new System.EventHandler(this.StartStopListenBtnClicked);
      // 
      // _middlePanel
      // 
      this._middlePanel.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
         | System.Windows.Forms.AnchorStyles.Left) 
         | System.Windows.Forms.AnchorStyles.Right);
      this._middlePanel.Controls.AddRange(new System.Windows.Forms.Control[] {
                                                                                this._logTextArea});
      this._middlePanel.Location = new System.Drawing.Point(0, 45);
      this._middlePanel.Name = "_middlePanel";
      this._middlePanel.Size = new System.Drawing.Size(540, 266);
      this._middlePanel.TabIndex = 1;
      // 
      // _logTextArea
      // 
      this._logTextArea.BackColor = System.Drawing.SystemColors.Window;
      this._logTextArea.Cursor = System.Windows.Forms.Cursors.Default;
      this._logTextArea.Dock = System.Windows.Forms.DockStyle.Fill;
      this._logTextArea.Font = new System.Drawing.Font("Courier New", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
      this._logTextArea.ForeColor = System.Drawing.SystemColors.WindowText;
      this._logTextArea.MaxLength = 0;
      this._logTextArea.Multiline = true;
      this._logTextArea.Name = "_logTextArea";
      this._logTextArea.ReadOnly = true;
      this._logTextArea.ScrollBars = System.Windows.Forms.ScrollBars.Both;
      this._logTextArea.Size = new System.Drawing.Size(540, 266);
      this._logTextArea.TabIndex = 0;
      this._logTextArea.TabStop = false;
      this._logTextArea.Text = "";
      this._logTextArea.WordWrap = false;
      // 
      // _topPanel
      // 
      this._topPanel.Controls.AddRange(new System.Windows.Forms.Control[] {
                                                                             this._configurationLabel,
                                                                             this._configurationCmbBox});
      this._topPanel.Dock = System.Windows.Forms.DockStyle.Top;
      this._topPanel.Name = "_topPanel";
      this._topPanel.Size = new System.Drawing.Size(542, 45);
      this._topPanel.TabIndex = 0;
      // 
      // _configurationLabel
      // 
      this._configurationLabel.Location = new System.Drawing.Point(35, 12);
      this._configurationLabel.Name = "_configurationLabel";
      this._configurationLabel.Size = new System.Drawing.Size(120, 24);
      this._configurationLabel.TabIndex = 0;
      this._configurationLabel.Text = "Current Configuration:";
      this._configurationLabel.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
      // 
      // _configurationCmbBox
      // 
      this._configurationCmbBox.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
      this._configurationCmbBox.Location = new System.Drawing.Point(180, 12);
      this._configurationCmbBox.Name = "_configurationCmbBox";
      this._configurationCmbBox.Size = new System.Drawing.Size(300, 21);
      this._configurationCmbBox.Sorted = false;
      this._configurationCmbBox.TabIndex = 1;
      this._configurationCmbBox.SelectedIndexChanged += new System.EventHandler(this.ConfigurationSelectedIndexChanged);
      // 
      // llpServer
      // 
      this.llpServer.ClientErrorReceived += new iNTERFACEWARE.Chameleon.LlpServer.ClientErrorReceivedEvent(this.llpServer_ClientErrorReceived);
      this.llpServer.ErrorReceived += new iNTERFACEWARE.Chameleon.LlpServer.ErrorReceivedEvent(this.llpServer_ErrorReceived);
      this.llpServer.MessageReceived += new iNTERFACEWARE.Chameleon.LlpServer.MessageReceivedEvent(this.llpServer_MessageReceived);
      this.llpServer.ClientConnected += new iNTERFACEWARE.Chameleon.LlpServer.ClientConnectedEvent(this.llpServer_ClientConnected);
      this.llpServer.DataIgnored += new iNTERFACEWARE.Chameleon.LlpServer.DataIgnoredEvent(this.llpServer_DataIgnored);
      this.llpServer.ClientDisconnected += new iNTERFACEWARE.Chameleon.LlpServer.ClientDisconnectedEvent(this.llpServer_ClientDisconnected);
      // 
      // ServerForm
      // 
      this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
      this.ClientSize = new System.Drawing.Size(542, 369);
      this.Controls.AddRange(new System.Windows.Forms.Control[] {
                                                                   this._topPanel,
                                                                   this._middlePanel,
                                                                   this._bottomPanel});
      this.Name = "ServerForm";
      this.Text = "Server";
      this.Closing += new System.ComponentModel.CancelEventHandler(this.ServerForm_Closing);
      this.Load += new System.EventHandler(this.ServerForm_Load);
      this._bottomPanel.ResumeLayout(false);
      this._middlePanel.ResumeLayout(false);
      this._topPanel.ResumeLayout(false);
      this.ResumeLayout(false);

   }
#endregion

	private void ServerForm_Closing(object sender, System.ComponentModel.CancelEventArgs e)
	{
		if (llpServer.IsListening)
		{
			llpServer.StopListen();
			llpServer.DisconnectAllClients();
		}	
		// We must manually call 'dispose' to avoid an exception on exit.
		llpServer.Dispose();
	}

	// This is the event we will get when a Client is connected to the Server.
	// The 'Client' object (of type LlpConnection) can be used to interact with the machine that is sending
	// the data.  There can be multiple clients connected to the server.
	private void llpServer_ClientConnected(object sender, System.EventArgs e, iNTERFACEWARE.Chameleon.LlpConnection client)
	{
		Println("Client Connected from :" + client.RemoteIpAddressAsString);
	}

	// This is the event we will get when a Client disconnects from the server. Note that sometimes
	// if the client is not well behaved we may not receive this event.
	private void llpServer_ClientDisconnected(object sender, System.EventArgs e, iNTERFACEWARE.Chameleon.LlpConnection client)
	{
		Println("Client Disconnected from :" + client.RemoteIpAddressAsString );
	}

	// This is the event we'll get when we have an error on a given TCP/IP connection.
	private void llpServer_ClientErrorReceived(object sender, System.EventArgs e, iNTERFACEWARE.Chameleon.LlpConnection client, string errorMessage, int errorCode)
	{
		Println(engine.MakeMessagePrintable("Client Error Received : " + errorMessage) );
	}

	// This event fires when we receive data that is not using the Lower Layer Protocol (LLP).
	// LLP is the standard protocol for sending HL7 messages, and is used by Chameleon.  See the manual
	// for more details.
	private void llpServer_DataIgnored(object sender, System.EventArgs e, iNTERFACEWARE.Chameleon.LlpConnection client, string data)
	{
		Println("Ignored Data outside of LLP protocol received from :" + client.RemoteIpAddressAsString );
		Println(engine.MakeMessagePrintable(data ));
	}

	// This event fires if we have an error related to the listening socket itself rather
	// than a specific connection.  In practice this event is rarely invoked.
	private void llpServer_ErrorReceived(object sender, System.EventArgs e, string errorMessage, int code)
	{
		Println(engine.MakeMessagePrintable("Error Received : " + errorMessage));
	}

	// This event is the main entry point when we get an HL7 message from a connected 
	// peer.  We invoke the ParseMessage method on the hl7Engine object.  
	//
	// Note that the LLP TCP/IP transport components are not tightly coupled to the
	// hl7Engine object.  You can use any kind of transport you like with Chameleon, including
	// your own transports or third party components.
	//
	// Notice that in this application we do not block on events - this is because the
	// networking components are asynchronous.  You can write synchronous applications using
	// Chameleon but you would need to use a transport mechanism other than the LLP TCP/IP
	// components supplied here.
	private void llpServer_MessageReceived(object sender, System.EventArgs e, iNTERFACEWARE.Chameleon.LlpConnection client, string message)
	{
		Println("Received a message from " + client.RemoteIpAddressAsString + ":");
		// The MakeMessagePrintable method is used to make the HL7 message to a format that
		// can be easily read in the log files.
		// In the Client example, you'll find the usage of this method on the LlpClient object.
		Println(engine.MakeMessagePrintable(message));

		string ackMessage = "";
		try
		{  
			// We manually invoke the engine to parse the message.  The second
			// parameter is set to 'null'; it used to be part of the transport framework for
			// Chameleon, but is no longer required.
			engine.ParseMessage(message, null);
		
			// Now we need to generate the ack and send it back to the user.
			// We use auto-acknowledgement; to obtain more information regarding auto-acknowledgement,
			// please read the Chameleon manual.

			//this method generates an Ack using the ackEngine
			ackMessage = ackEngine.GenerateAckFromMessage(message);
		}
		catch (ChameleonException ex)
		{
			// An error occured! Send back a negative-acknowledgement
			Println(engine.MakeMessagePrintable(ex.ToString() + " while trying to generate ACK.  So we are sending a NACK."));
			try
			{
				ackMessage = nackEngine.GenerateAckFromMessage(message);
			}
			catch (ChameleonException exception)
			{
				// Just in case generating the Nack fails, we'll send a hard-coded nack instead.
				Println("Generating Nack Failed! Sending a hard-coded Nack.");
                Println(exception.Message);
				ackMessage = "MSH|^~\\&|ACME|ACME_APP|XYZ|XYZ_APP|||NACK|XXX|P|2.4|\rMSA|AA|XXX|\r";
			}
		}
		finally
		{
			//we send the ACK/NACK message
			client.SendMessage(ackMessage);
		}		
	}
}

